///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2006-2009 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		GF	Jul 18, 2006	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.control;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Vector;

import com.tietoenator.trip.jxp.TdbControlObject;
import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.control.TdbControlObjectList;
import com.tietoenator.trip.jxp.control.TdbDatabaseAccessList;
import com.tietoenator.trip.jxp.control.TdbDatabaseFieldList;
import com.tietoenator.trip.jxp.control.TdbDatabaseList;
import com.tietoenator.trip.jxp.control.TdbEntryFormList;
import com.tietoenator.trip.jxp.control.TdbFileManagerList;
import com.tietoenator.trip.jxp.control.TdbGroupAccessList;
import com.tietoenator.trip.jxp.control.TdbGroupList;
import com.tietoenator.trip.jxp.control.TdbManagerList;
import com.tietoenator.trip.jxp.control.TdbOutputFormatList;
import com.tietoenator.trip.jxp.control.TdbOwnedDatabaseList;
import com.tietoenator.trip.jxp.control.TdbProcedureList;
import com.tietoenator.trip.jxp.control.TdbSearchFormList;
import com.tietoenator.trip.jxp.control.TdbUserAccessList;
import com.tietoenator.trip.jxp.control.TdbUserList;
import com.tietoenator.trip.jxp.control.TdbUserManagerList;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.grid.TdbGridClusterList;
import com.tietoenator.trip.jxp.grid.TdbGridHostList;
import com.tietoenator.trip.jxp.grid.TdbGridReplicaSetList;
import com.tietoenator.trip.jxp.session.TdbInterfaceType;
import com.tietoenator.trip.jxp.session.TdbSession;

/**
 * Very generic example of how to generate lists from CONTROL, such as
 * would be shown by CCL commands such as "Show BASe LISt", etc. The result of
 * each list operation is a Vector of TdbControlObject objects that can be
 * queried for their properties and used in further operations as input to the
 * various constructors of functional classes such as TdbDatabaseDesign, etc.
 */
@SuppressWarnings({"rawtypes","unchecked"})
public class ListGenerator
{

    private Vector<ListInfo> m_lists;
    private TdbSession m_session;

    /**
     * Information about a particular type of list generator -- each type
     * of CONTROL object list is one of these, e.g. database list, user
     * list, etc. Depending on the setting of m_args, the outer class can
     * prompt for a varying number of arguments.
     */
    private class ListInfo
    {

        /**
         * Class of list generator, used to create new instances
         */
        public Class m_list;
        /**
         * Name of list generator, used to prompt user
         */
        public String m_name;
        /**
         * Name of argument required to create list generator, used to prompt user
         */
        public String m_arg;
        /**
         * Boolean argument
         */
        public Boolean m_arg2;


        /**
         * Constructor -- create a ClassInfo without a named argument -- this corresponds
         * to a list generator that has the constructor format: TdbXxxList(TdbSession)
         *
         * @param c Class of list generator to create
         * @param n Name of list generator
         */
        public ListInfo(Class c, String n)
        {
            this(c, n, null);
        }

        /**
         * Constructor -- create an object with a named argument -- this corresponds to
         * a list generator that has the constructor format: TdbXxxList(TdbSession, String)
         *
         * @param c Class of list generator
         * @param n Name of list generator
         * @param a Argument to use when creating list generator
         */
        public ListInfo(Class c, String n, String a)
        {
            m_list = c;
            m_name = n;
            m_arg = a;
            m_arg2 = null;
        }

        /**
         * Constructor -- create an object with a named argument -- this corresponds to
         * a list generator that has the constructor format: TdbXxxList(TdbSession, String, boolean)
         *
         * @param c Class of list generator
         * @param n Name of list generator
         * @param a Argument to use when creating list generator
         * @param b Additional argument to use when creating list generator
         */
        public ListInfo(Class c, String n, String a, boolean b)
        {
            m_list = c;
            m_name = n;
            m_arg = a;
            m_arg2 = new Boolean(b);
        }
    };

    /**
     * Constructor
     * @throws TdbException
     */
    public ListGenerator()
            throws TdbException
    {
        m_session = SessionFactory.newInstance("**** TRIPjxp List Producer ****", true);
        m_lists = new Vector<ListInfo>();
        if (m_session.getSessionType() == TdbInterfaceType.GRID)
        {
            populateGridLists();
        }
        else
        {
            populatePhysicalLists();
        }
    }

    /**
     * Main loop of the program, this prompts the user to choose a list generator from the available
     * types (depending on connection type) and then executes the chosen generator. Due to the
     * generality of approach, the generator is created using Reflection rather than being created
     * directly.
     *
     * @throws TdbException
     */
    public void mainLoop()
            throws TdbException
    {
        Console stdin = new Console();
        int ix;
        boolean useOwner = false;

        for (;;)
        {
            // Show the available list generators
            ix = 1;
            System.out.println("\nAvailable Generators\n====================\n");
            System.out.println("0.\tExit");
            for (ListInfo linf : m_lists)
            {
                System.out.println(ix + ".\t" + linf.m_name + " List");
                ix++;
            }

            // Prompt the user to choose a generator
            System.out.println("");
            ix = stdin.readint("List generator", 0, true);
            if (ix < 0 || ix > m_lists.size())
            {
                continue;
            }
            if (ix == 0)
            {
                break;
            }

            useOwner = (ix == 6 || ix == 13) ? true : false;


            // Prompt for an argument, if required by the list type
            ListInfo linf = m_lists.elementAt(ix - 1);
            String arg = null;
            if (linf.m_arg != null)
            {
                arg = stdin.readln(linf.m_arg, null);
            }

            // Create a list generator of the requested type
            TdbControlObjectList list = createList(linf, arg);

            // Show the result
            ListPrinter.showHeader(useOwner);
            for (TdbControlObject obj : list)
            {
                ListPrinter.showItem(obj, useOwner);
            }
        }
        m_session.logout();
    }

    // create a list of generators specific to TRIPgrid servers
    private void populateGridLists()
    {
        m_lists.add(new ListInfo(TdbGridClusterList.class, "Cluster"));
        m_lists.add(new ListInfo(TdbGridHostList.class, "Host"));
        m_lists.add(new ListInfo(TdbGridReplicaSetList.class, "Replica Set"));
    }

    // create a list of generators specific to TRIPnet / TRIPweb servers
    private void populatePhysicalLists()
    {
        m_lists.add(new ListInfo(TdbDatabaseAccessList.class, "Database Access", "Database"));
        m_lists.add(new ListInfo(TdbDatabaseFieldList.class, "Database Field", "Database"));
        m_lists.add(new ListInfo(TdbDatabaseList.class, "Database"));
        m_lists.add(new ListInfo(TdbEntryFormList.class, "Entry Form", "Database"));
        m_lists.add(new ListInfo(TdbFileManagerList.class, "File Manager"));
        m_lists.add(new ListInfo(TdbGroupAccessList.class, "Group Access", "Group"));
        m_lists.add(new ListInfo(TdbGroupList.class, "Group"));
        m_lists.add(new ListInfo(TdbManagerList.class, "Manager"));
        m_lists.add(new ListInfo(TdbOutputFormatList.class, "Report", "Database"));
        m_lists.add(new ListInfo(TdbOwnedDatabaseList.class, "Owned Database"));
        m_lists.add(new ListInfo(TdbProcedureList.class, "Procedure", "User or Group"));
        m_lists.add(new ListInfo(TdbSearchFormList.class, "Search Form"));
        m_lists.add(new ListInfo(TdbUserAccessList.class, "Explicit User Access", "User"));
        m_lists.add(new ListInfo(TdbUserList.class, "User"));
        m_lists.add(new ListInfo(TdbUserManagerList.class, "User Manager"));
        m_lists.add(new ListInfo(TdbUserAccessList.class, "Total User Access", "User", true));
    }

    // Uses Reflection to create an instance of the required list generator as identified
    // by the ListInfo passed in.
    private TdbControlObjectList createList(ListInfo inf, String arg)
            throws TdbException
    {
        TdbControlObjectList l = null;
        Constructor ctor;
        Object[] args;
        Class[] params;

        try
        {
            if (inf.m_arg != null && inf.m_arg2 != null && arg != null)
            {
                // TdbControlObjectList(TdbSession, String, boolean)
                params = new Class[]
                        {
                            TdbSession.class, String.class, boolean.class
                        };

                args = new Object[]
                        {
                            m_session, arg, inf.m_arg2.booleanValue()
                        };

                try
                {
                    ctor = inf.m_list.getConstructor(params);
                    l = (TdbControlObjectList) ctor.newInstance(args);
                }
                catch (NoSuchMethodException nsme)
                {
                    // Constructor not found, fallback to using TdbControlObjectList(TdbSession)
                    params = new Class[]
                            {
                                TdbSession.class
                            };
                    args = new Object[]
                            {
                                m_session
                            };

                    System.out.println("Warning: required constructor not found -- using default");

                    ctor = inf.m_list.getConstructor(params);
                    l = (TdbControlObjectList) ctor.newInstance(args);
                }
            }
            else if (inf.m_arg != null && arg != null)
            {
                // TdbControlObjectList(TdbSession, String)
                params = new Class[]
                        {
                            TdbSession.class, String.class
                        };

                args = new Object[]
                        {
                            m_session, arg
                        };

                try
                {
                    ctor = inf.m_list.getConstructor(params);
                    l = (TdbControlObjectList) ctor.newInstance(args);
                }
                catch (NoSuchMethodException nsme)
                {
                    // Constructor not found, fallback to using TdbControlObjectList(TdbSession)
                    params = new Class[]
                            {
                                TdbSession.class
                            };
                    args = new Object[]
                            {
                                m_session
                            };

                    System.out.println("Warning: required constructor not found -- using default");

                    ctor = inf.m_list.getConstructor(params);
                    l = (TdbControlObjectList) ctor.newInstance(args);
                }
            }
            else
            {
                // TdbControlObjectList(TdbSession)
                params = new Class[]
                        {
                            TdbSession.class
                        };

                args = new Object[]
                        {
                            m_session
                        };

                ctor = inf.m_list.getConstructor(params);
                l = (TdbControlObjectList) ctor.newInstance(args);
            }
        }
        catch (Exception ex)
        {
            Throwable ct;

            // If the exception is thrown during construction, dig into
            // what the exception actually is and pull out any embedded
            // TdbException to throw that directly
            if (ex instanceof InvocationTargetException)
            {
                ct = ex.getCause();
                if (ct != null && ct instanceof TdbException)
                {
                    throw (TdbException) ct;
                }
            }

            // Otherwise, wrap whatever happened in a TdbException (for
            // generality) and throw it
            throw new TdbException(TdbException.GENERAL_ERROR, ex);
        }

        // Return the new TdbControlObjectList object
        return l;
    }

    /**
     * Main entry point
     * @param args Not used
     */
    public static void main(String[] args)
    {
        try
        {
            ListGenerator l = new ListGenerator();
            l.mainLoop();
        }
        catch (TdbException e)
        {
            System.out.println(e.getDebugMessage());
        }
    }
}
