///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2009 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	2008-03-26	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.data;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.data.TdbComponent;
import com.tietoenator.trip.jxp.data.TdbField;
import com.tietoenator.trip.jxp.data.TdbRecord;
import com.tietoenator.trip.jxp.data.TdbRecordSet;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.session.TdbSession;

/**
 * This class demonstrates a simple means of placing a request to a server for
 * a set of records from a cluster, optionally conforming to a specified query.
 * The set of records retrieved are displayed on the console.
 */
public class ClusterSearch
{

   private TdbSession m_session;

   /**
    * Constructor
    * @throws TdbException
    */
   public ClusterSearch()
           throws TdbException
   {
      // Create a session on a physical server (no grid support)
      m_session = SessionFactory.newInstance("**** Cluster search example ****", false);
   }

   /**
    * Retrieve a range of records from the named database, optionally using
    * a query to subset the database, picking the record range as specified
    *
    * @param qry (optional) query
    * @param from First record # to be retrieved
    * @param to Last record # to be retrieved
    * @throws TdbException
    */
   public void getRecords(String qry, int from, int to, boolean reversed)
           throws TdbException
   {
      TdbRecordSet rs = new TdbRecordSet(m_session);

      // Set the general parameters for the retrieval
      rs.setDatabase("LEWIS=ALICE,CARROLL");
      rs.setQuery(qry);
      rs.setFrom(from);
      rs.setTo(to);
      rs.setRetrieveReversed(reversed);

      // Create a "template" for the fields the result should contain
      TdbRecord tmpl = new TdbRecord(m_session);
      tmpl.addToTemplate("CHAPTNR");
      tmpl.addToTemplate("CHAPTER");
      tmpl.addToTemplate("PERSON");
      tmpl.addToTemplate("SPEAKER");
      tmpl.addToTemplate("TXT");
      tmpl.setTemplateMarkup(">>", "<<");
      rs.setRetrievalTemplate(tmpl);

      // Retrieve the records from the server -- this is the only
      // method call that actually hits the network in any way
      rs.get();

      // Use the simplest method of coercing records to strings...
      for (TdbRecord record : rs.records())
      {
         // First the head record structure
         System.out.println("\nRecord " + record.getRecordId() + "\n");
         for (TdbField field : record.getHead().fields())
         {
            System.out.println(field.getName() + ": " + field.toString());
         }

         // Then any part records
         for (TdbComponent part : record.parts())
         {
            System.out.println("\nPart " + part.getId());
            for (TdbField field : part.fields())
            {
               System.out.println(field.getName() + ": " + field.toString());
            }
         }
      }

      // Terminate the session
      m_session.logout();
   }

   /**
    * Main entry point
    * @param args Not used
    */
   public static void main(String[] args)
   {
      try
      {
         ClusterSearch rs = new ClusterSearch();

         Console stdin = new Console();
         String qry = null;
         int from = 0, to = 0;
         boolean rev = false;

         System.out.println("Using a CLUSTER over ALICE and CARROLL");

         // Prompt for an optional query
         qry = stdin.readln("Query (optional)", null);

         // Prompt for the retrieval range
         from = stdin.readint("From (1)", 1, true);
         to = stdin.readint("To (10)", 10, true);

         // Prompt for reverse retrieval
         if (stdin.readint("Reverse retrieval? (1|0)", 0, true) != 0)
         {
            rev = true;
         }

         // Perform the retrieval
         rs.getRecords(qry, from, to, rev);
      }
      catch (TdbException e)
      {
         System.out.println(e.getDebugMessage());
      }
   }
}
